@@ -1,12 +1,16 @@
 """Convert inputs to arrays with at least one dimension."""
+from __future__ import annotations
+from typing import List, Union
+
 import numpy
 import numpoly
 
+from ..baseclass import ndpoly, PolyLike
 from ..dispatch import implements
 
 
 @implements(numpy.atleast_1d)
-def atleast_1d(*arys):
+def atleast_1d(*arys: PolyLike) -> Union[ndpoly, List[ndpoly]]:
     """
     Convert inputs to arrays with at least one dimension.
 
@@ -14,13 +18,12 @@ def atleast_1d(*arys):
     higher-dimensional inputs are preserved.
 
     Args:
-        arys (numpoly.ndpoly):
+        arys:
             One or more input arrays.
 
     Returns:
-        (ndarray):
-            An array, or list of arrays, each with ``a.ndim >= 1``. Copies are
-            made only if necessary.
+        An array, or list of arrays, each with ``a.ndim >= 1``.
+        Copies are made only if necessary.
 
     Examples:
         >>> numpoly.atleast_1d(numpoly.variable())
