@@ -1,30 +1,33 @@
 """Broadcast any number of arrays against each other."""
+from __future__ import annotations
+from typing import Any, List
+
 import numpy
 import numpoly
 
+from ..baseclass import ndpoly, PolyLike
 from ..dispatch import implements
 
 
 @implements(numpy.broadcast_arrays)
-def broadcast_arrays(*args, **kwargs):
+def broadcast_arrays(*args: PolyLike, **kwargs: Any) -> List[ndpoly]:
     """
     Broadcast any number of arrays against each other.
 
     Args:
-        args (numpoly.ndpoly):
+        args:
             The arrays to broadcast.
-        subok (bool):
+        subok:
             If True, then sub-classes will be passed-through, otherwise the
             returned arrays will be forced to be a base-class array (default).
 
     Returns:
-        (List[numpoly.ndpoly]):
-            These arrays are views on the original arrays.  They are typically
-            not contiguous.  Furthermore, more than one element of a
-            broadcasted array may refer to a single memory location. If you
-            need to write to the arrays, make copies first. While you can set
-            the ``writable`` flag True, writing to a single output value may
-            end up changing more than one location in the output array.
+        These arrays are views on the original arrays.  They are typically not
+        contiguous.  Furthermore, more than one element of a broadcasted array
+        may refer to a single memory location. If you need to write to the
+        arrays, make copies first. While you can set the ``writable`` flag
+        True, writing to a single output value may end up changing more than
+        one location in the output array.
 
     Examples:
         >>> poly1 = numpoly.monomial(3).reshape(1, 3)
@@ -43,7 +46,7 @@ def broadcast_arrays(*args, **kwargs):
                     [q0, q0, q0]])
 
     """
-    args = [numpoly.aspolynomial(arg) for arg in args]
-    results = numpy.broadcast_arrays(*[arg.values for arg in args], **kwargs)
+    args_ = [numpoly.aspolynomial(arg) for arg in args]
+    results = numpy.broadcast_arrays(*[arg.values for arg in args_], **kwargs)
     return [numpoly.aspolynomial(result, names=arg.indeterminants)
-            for result, arg in zip(results, args)]
+            for result, arg in zip(results, args_)]
