@@ -1,19 +1,28 @@
 """Split an array into multiple sub-arrays."""
+from __future__ import annotations
+from typing import List
+
 import numpy
+import numpy.typing
 import numpoly
 
+from ..baseclass import ndpoly, PolyLike
 from ..dispatch import implements
 
 
 @implements(numpy.split)
-def split(ary, indices_or_sections, axis=0):
+def split(
+    ary: PolyLike,
+    indices_or_sections: numpy.typing.ArrayLike,
+    axis: int = 0,
+) -> List[ndpoly]:
     """
     Split an array into multiple sub-arrays.
 
     Args:
-        ary (numpoly.ndpoly):
+        ary:
             Array to be divided into sub-arrays.
-        indices_or_sections (int, Sequence[int]):
+        indices_or_sections:
             If `indices_or_sections` is an integer, N, the array will be
             divided into N equal arrays along `axis`.  If such a split is not
             possible, an error is raised.
@@ -28,12 +37,11 @@ def split(ary, indices_or_sections, axis=0):
 
             If an index exceeds the dimension of the array along `axis`, an
             empty sub-array is returned correspondingly.
-        axis (int):
+        axis:
             The axis along which to split, default is 0.
 
     Returns:
-        (List[numpoly.ndpoly])
-            A list of sub-arrays.
+        A list of sub-arrays.
 
     Raises:
         ValueError:
@@ -55,20 +63,20 @@ def split(ary, indices_or_sections, axis=0):
     Examples:
         >>> poly = numpoly.monomial(16).reshape(4, 4)
         >>> poly
-        polynomial([[1, q, q**2, q**3],
-                    [q**4, q**5, q**6, q**7],
-                    [q**8, q**9, q**10, q**11],
-                    [q**12, q**13, q**14, q**15]])
+        polynomial([[1, q0, q0**2, q0**3],
+                    [q0**4, q0**5, q0**6, q0**7],
+                    [q0**8, q0**9, q0**10, q0**11],
+                    [q0**12, q0**13, q0**14, q0**15]])
         >>> part1, _ = numpoly.split(poly, 2, axis=0)
         >>> part1
-        polynomial([[1, q, q**2, q**3],
-                    [q**4, q**5, q**6, q**7]])
+        polynomial([[1, q0, q0**2, q0**3],
+                    [q0**4, q0**5, q0**6, q0**7]])
         >>> part1, _ = numpoly.split(poly, 2, axis=1)
         >>> part1
-        polynomial([[1, q],
-                    [q**4, q**5],
-                    [q**8, q**9],
-                    [q**12, q**13]])
+        polynomial([[1, q0],
+                    [q0**4, q0**5],
+                    [q0**8, q0**9],
+                    [q0**12, q0**13]])
 
     """
     ary = numpoly.aspolynomial(ary)
