@@ -1,27 +1,37 @@
 """Subtract arguments, element-wise."""
-import numpy
-import numpoly
+from __future__ import annotations
+from typing import Any, Optional
 
+import numpy
+import numpy.typing
+
+from ..baseclass import ndpoly, PolyLike
 from ..dispatch import implements, simple_dispatch
 
 
 @implements(numpy.subtract)
-def subtract(x1, x2, out=None, where=True, **kwargs):
+def subtract(
+    x1: PolyLike,
+    x2: PolyLike,
+    out: Optional[ndpoly] = None,
+    where: numpy.typing.ArrayLike = True,
+    **kwargs: Any,
+) -> ndpoly:
     """
     Subtract arguments, element-wise.
 
     Args:
-        x1, x2 (numpoly.ndpoly):
+        x1, x2:
             The arrays to be subtracted from each other. If
             ``x1.shape != x2.shape``, they must be broadcastable to a common
             shape (which becomes the shape of the output).
-        out (Optional[numpy.ndarray]):
+        out:
             A location into which the result is stored. If provided, it must
             have a shape that the inputs broadcast to. If not provided or
             `None`, a freshly-allocated array is returned. A tuple (possible
             only as a keyword argument) must have length equal to the number of
             outputs.
-        where (Union[bool, numpy.ndarray]):
+        where:
             This condition is broadcast over the input. At locations where the
             condition is True, the `out` array will be set to the ufunc result.
             Elsewhere, the `out` array will retain its original value. Note
@@ -32,9 +42,8 @@ def subtract(x1, x2, out=None, where=True, **kwargs):
             Keyword args passed to numpy.ufunc.
 
     Returns:
-        (numpoly.ndpoly):
-            The difference of `x1` and `x2`, element-wise. This is a scalar if
-            both `x1` and `x2` are scalars.
+        The difference of `x1` and `x2`, element-wise.
+        This is a scalar if both `x1` and `x2` are scalars.
 
     Examples:
         >>> q0, q1 = numpoly.variable(2)
@@ -51,7 +60,7 @@ def subtract(x1, x2, out=None, where=True, **kwargs):
     return simple_dispatch(
         numpy_func=numpy.subtract,
         inputs=(x1, x2),
-        out=out,
+        out=None if out is None else (out,),
         where=where,
         **kwargs
     )
