@@ -1,7 +1,8 @@
 """Functions for constructing polynomials."""
 from .polynomial import polynomial
 from .aspolynomial import aspolynomial
-from .clean import clean_attributes
+from .clean import (
+    clean_attributes, remove_redundant_coefficients, remove_redundant_names)
 from .from_attributes import polynomial_from_attributes
 from .from_roots import polynomial_from_roots
 from .monomial import monomial
