@@ -1,15 +1,22 @@
 """Attempt to cast an object into a polynomial array."""
+from __future__ import annotations
+from typing import Optional, Tuple, Union
+
 import numpy
+import numpy.typing
+
 import numpoly
 
 from .compose import compose_polynomial_array
+from ..baseclass import ndpoly, PolyLike
 
 
 def polynomial(
-        poly_like=None,
-        names=None,
-        dtype=None,
-):
+        poly_like: PolyLike = 0,
+        names: Union[None, str, Tuple[str, ...], ndpoly] = None,
+        dtype: Optional[numpy.typing.DTypeLike] = None,
+        allocation: Optional[int] = None,
+) -> ndpoly:
     """
     Attempt to cast an object into a polynomial array.
 
@@ -30,46 +37,39 @@ def polynomial(
     ==================  =======================================================
 
     Args:
-        poly_like (typing.Any):
+        poly_like:
             Input to be converted to a `numpoly.ndpoly` polynomial type.
-        names (str, typing.Tuple[str, ...]):
+        names:
             Name of the indeterminant variables. If possible to infer from
             ``poly_like``, this argument will be ignored.
-        dtype (type, numpy.dtype):
+        dtype:
             Data type used for the polynomial coefficients.
+        allocation:
+            The maximum number of polynomial exponents. If omitted, use
+            length of exponents for allocation.
 
     Returns:
-        (numpoly.ndpoly):
-            Polynomial based on input ``poly_like``.
+        Polynomial based on input ``poly_like``.
 
     Examples:
         >>> numpoly.polynomial({(1,): 1})
-        polynomial(q)
-        >>> x, y = numpoly.symbols("x y")
-        >>> x**2 + x*y + 2
-        polynomial(x*y+x**2+2)
-        >>> -3*x + x**2 + y
-        polynomial(x**2+y-3*x)
-        >>> numpoly.polynomial([x*y, x, y])
-        polynomial([x*y, x, y])
+        polynomial(q0)
+        >>> q0, q1 = numpoly.variable(2)
+        >>> q0**2+q0*q1+2
+        polynomial(q0*q1+q0**2+2)
+        >>> -3*q0+q0**2+q1
+        polynomial(q0**2+q1-3*q0)
+        >>> numpoly.polynomial([q0*q1, q0, q1])
+        polynomial([q0*q1, q0, q1])
         >>> numpoly.polynomial([1, 2, 3])
         polynomial([1, 2, 3])
         >>> import sympy
-        >>> x_, y_ = sympy.symbols("x, y")
-        >>> numpoly.polynomial(3*x_*y_ - 4 + x_**5)
-        polynomial(x**5+3*x*y-4)
+        >>> q0_, q1_ = sympy.symbols("q0, q1")
+        >>> numpoly.polynomial(3*q0_*q1_-4+q0_**5)
+        polynomial(q0**5+3*q0*q1-4)
 
     """
-    if poly_like is None:
-        poly = numpoly.ndpoly(
-            exponents=[(0,)],
-            shape=(),
-            names=names,
-            dtype=dtype,
-        )
-        poly[";"] = 0
-
-    elif isinstance(poly_like, dict):
+    if isinstance(poly_like, dict):
         poly = numpoly.ndpoly(exponents=[(0,)], shape=())
         exponents, coefficients = zip(*list(poly_like.items()))
         poly = numpoly.ndpoly.from_attributes(
@@ -77,6 +77,7 @@ def polynomial(
             coefficients=coefficients,
             names=names,
             dtype=dtype,
+            allocation=allocation,
         )
 
     elif isinstance(poly_like, numpoly.ndpoly):
@@ -87,13 +88,17 @@ def polynomial(
             coefficients=poly_like.coefficients,
             names=names,
             dtype=dtype,
+            allocation=allocation,
         )
 
     # assume polynomial converted to structured array
     elif isinstance(poly_like, numpy.ndarray) and poly_like.dtype.names:
 
         keys = numpy.asarray(poly_like.dtype.names, dtype="U")
-        exponents = keys.flatten().view(numpy.uint32)-numpoly.ndpoly.KEY_OFFSET
+        keys = numpy.array([key for key in keys if not key.isdigit()])
+        keys = numpy.array(
+            keys, dtype=f"U{numpy.max(numpy.char.str_len(keys))}")
+        exponents = keys.view(numpy.uint32)-numpoly.ndpoly.KEY_OFFSET
         exponents = exponents.reshape(len(keys), -1)
 
         coefficients = [poly_like[key] for key in poly_like.dtype.names]
@@ -101,33 +106,39 @@ def polynomial(
             exponents=exponents,
             coefficients=coefficients,
             names=names,
+            allocation=allocation,
         )
 
     elif isinstance(poly_like, (int, float, numpy.ndarray, numpy.generic)):
         poly = numpoly.ndpoly.from_attributes(
             exponents=[(0,)],
-            coefficients=numpy.array([poly_like]),
+            coefficients=[numpy.asarray(poly_like)],
             names=names,
             dtype=dtype,
+            allocation=allocation,
         )
 
     # handler for sympy objects
     elif hasattr(poly_like, "as_poly"):
-        poly_like = poly_like.as_poly()
-        exponents = poly_like.monoms()
-        coefficients = [int(coeff) if coeff.is_integer else float(coeff)
-                        for coeff in poly_like.coeffs()]
-        names = [str(elem) for elem in poly_like.gens]
+        poly_like = poly_like.as_poly()  # type: ignore
+        exponents = poly_like.monoms()  # type: ignore
+        coefficients = [
+            int(coeff) if coeff.is_integer else float(coeff)  # type: ignore
+            for coeff in poly_like.coeffs()  # type: ignore
+        ]
+        names = [str(elem) for elem in poly_like.gens]  # type: ignore
         poly = numpoly.ndpoly.from_attributes(
             exponents=exponents,
             coefficients=coefficients,
             names=names,
+            allocation=allocation,
         )
 
     else:
         poly = compose_polynomial_array(
-            arrays=poly_like,
+            arrays=poly_like,  # type: ignore
             dtype=dtype,
+            allocation=allocation,
         )
 
     return poly
